

class InferenceUtils:
    
    @staticmethod
    def sequence_classification(texts):
        pass
    
    @staticmethod
    def question_answering():
        pass
    
    @staticmethod
    def summarization():
        pass
    
    @staticmethod
    def translation():
        pass
    
    @staticmethod
    def token_classification():
        pass